==> apeman-api-recover-1.0.3.tgz: package/lib/define.js <==
/**
 * Define api of apeman-api-recover
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const assert = require('assert')
const uuid = require('uuid')
const defaults = require('defaults')
const aslogger = require('aslogger')
const apemanApiSign = require('apeman-api-sign')
const describe = require('./describe')
const { moduleProxy } = require('apemanmock')
const apemanerror = require('apemanerror')
const {
  RESOURCE_NOT_FOUND_ERROR,
  TOKEN_WRONG_ERROR,
  TOKEN_EXPIRED_ERROR
} = apemanerror.ErrorCodes

/** @lends define */
function define (User, UserSign, UserRecover, options = {}) {
  assert.ok(UserRecover, 'UserRecover is required.')
  assert.ok(!!(UserRecover.$$inherited && UserRecover.$$inherited.ApUserRecover), 'UserRecover should inherit ApUserRecover.')
  let { passwordSpec, logging } = options
  let logger = aslogger({ disabled: !logging })

  let signModule = apemanApiSign(User, UserSign, {
    passwordSpec, logging
  })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    issue (email) {
      let { expireDuration } = defaults(options, {
        expireDuration: 26 * 60 * 60 * 10000
      })
      return co(function * () {
        let user = email && (yield User.findOne({ where: { email } }))
        if (!user) {
          throw apemanerror(RESOURCE_NOT_FOUND_ERROR)
        }
        let token = uuid.v4().replace(/\-/g, '')
        let userId = user.id
        let recover = (yield UserRecover.findOrCreate({
          where: { userId },
          defaults: { userId, token }
        }))[ 0 ]
        yield recover.update({
          token,
          expireAt: new Date(new Date().getTime() + expireDuration),
          done: false
        })
        return { token, expireDuration, userId }
      })
    },

    check (token) {
      const s = this
      return co(function * () {
        let recover = yield s._getRecover(token)
        return !!recover
      })
    },

    update (token, newPassword) {
      const s = this
      let { session, sign } = s
      return co(function * () {
        let recover = yield s._getRecover(token)
        let { userId } = recover
        let userSigns = (yield UserSign.findAll({
          where: { userId },
          include: [ UserSign.getRef('user') ]
        }))
        let userSign = userSigns[ 0 ]
        session.signed = apemanApiSign.formatSigned(userSign.user, userSign)
        let keys = userSigns.reduce((keys, userSign) => Object.assign(keys, {
          [userSign.kind]: userSign.key
        }), {})
        yield sign.signmod(keys, newPassword)
        yield sign.signout()
        yield recover.update({
          done: true,
          doneAt: new Date()
        })
        return { key: userSign.key, password: newPassword }
      })
    },

    /**
     * Get recover from token
     * @param {string} token - Token to search
     * @throws {Error} - Unless valid token found
     * @returns {Promise}
     */
    _getRecover (token) {
      return co(function * () {
        let recover = token && (yield UserRecover.findOne({ token }))
        if (!recover) {
          throw apemanerror(TOKEN_WRONG_ERROR)
        }
        if (recover.expired) {
          throw apemanerror(TOKEN_EXPIRED_ERROR)
        }
        if (recover.done) {
          throw apemanerror(TOKEN_EXPIRED_ERROR)
        }
        return recover
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        s.sign = moduleProxy(signModule, s)
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()

  }
}

module.exports = define

==> apeman-api-recover-1.0.6.tgz: package/lib/define.js <==
/**
 * Define api of apeman-api-recover
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const assert = require('assert')
const uuid = require('uuid')
const defaults = require('defaults')
const aslogger = require('aslogger')
const apemanApiSign = require('apeman-api-sign')
const describe = require('./describe')
const load = require('apeman-api-client/load')
const apemanerror = require('apemanerror')
const {
  RESOURCE_NOT_FOUND_ERROR,
  TOKEN_WRONG_ERROR,
  TOKEN_EXPIRED_ERROR
} = apemanerror.ErrorCodes

/** @lends define */
function define (User, UserSign, UserRecover, options = {}) {
  assert.ok(UserRecover, 'UserRecover is required.')
  assert.ok(!!(UserRecover.$$inherited && UserRecover.$$inherited.ApUserRecover), 'UserRecover should inherit ApUserRecover.')
  let { passwordSpec, logging } = options
  let logger = aslogger({ disabled: !logging })

  let signApi = apemanApiSign(User, UserSign, {
    passwordSpec, logging
  })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    issue (email) {
      let { expireDuration } = defaults(options, {
        expireDuration: 26 * 60 * 60 * 10000
      })
      return co(function * () {
        let user = email && (yield User.findOne({ where: { email } }))
        if (!user) {
          throw apemanerror(RESOURCE_NOT_FOUND_ERROR)
        }
        let token = uuid.v4().replace(/\-/g, '')
        let userId = user.id
        let recover = (yield UserRecover.findOrCreate({
          where: { userId },
          defaults: { userId, token }
        }))[ 0 ]
        yield recover.update({
          token,
          expireAt: new Date(new Date().getTime() + expireDuration)
        })
        return { token, expireDuration, userId }
      })
    },

    check (token) {
      const s = this
      return co(function * () {
        let recover = yield s._getRecover(token)
        return !!recover
      })
    },

    update (token, newPassword) {
      const s = this
      let { session } = s
      return co(function * () {
        let sign = yield load(signApi, { $$ctx: s })
        let recover = yield s._getRecover(token)
        let { userId } = recover
        let userSigns = (yield UserSign.findAll({
          where: { userId },
          include: [ UserSign.getRef('user') ]
        }))
        let userSign = userSigns[ 0 ]
        session.signed = apemanApiSign.formatSigned(userSign.user, userSign)
        let keys = userSigns.reduce((keys, userSign) => Object.assign(keys, {
          [userSign.kind]: userSign.key
        }), {})
        yield sign.signmod(keys, newPassword)
        yield sign.signout()
        yield recover.update({
          doneAt: new Date()
        })
        return { key: userSign.key, password: newPassword }
      })
    },

    /**
     * Get recover from token
     * @param {string} token - Token to search
     * @throws {Error} - Unless valid token found
     * @returns {Promise}
     */
    _getRecover (token) {
      return co(function * () {
        let recover = token && (yield UserRecover.findOne({ token }))
        if (!recover) {
          throw apemanerror(TOKEN_WRONG_ERROR)
        }
        if (recover.expired) {
          throw apemanerror(TOKEN_EXPIRED_ERROR)
        }
        if (recover.done) {
          throw apemanerror(TOKEN_EXPIRED_ERROR)
        }
        return recover
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()

  }
}

module.exports = define
